/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of this code is David Baum.
 * Portions created by David Baum are Copyright (C) 1998 David Baum.
 * All Rights Reserved.
 */

#include "BlockStmt.h"

BlockStmt::~BlockStmt()
{
	Stmt *s;
	
	while((s = fList.RemoveHead()) != nil)
		delete s;
}


bool BlockStmt::Check(CheckState &state)
{
	bool ok = true;
	fNullable = true;
	
	for(Stmt *s = fList.GetHead(); s; s=s->GetNext())
	{
		if (!s->Check(state)) ok = false;
		if (!s->IsNullable()) fNullable = false;
	}
	
	return ok;
}


void BlockStmt::Emit(Bytecode &b)
{
	for(Stmt *s = fList.GetHead(); s; s=s->GetNext())
	{
		s->Emit(b);
	}
}


BlockStmt* BlockStmt::Clone(Mapping *b) const
{
	BlockStmt *block = new BlockStmt;

	for(Stmt *s = fList.GetHead(); s; s=s->GetNext())
		block->fList.InsertTail(s->Clone(b));
	
	return block;	
}
