/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of this code is David Baum.
 * Portions created by David Baum are Copyright (C) 1998 David Baum.
 * All Rights Reserved.
 */

#ifndef __Fragment_h
#define __Fragment_h

#ifndef __PTypes_h
#include "PTypes.h"
#endif

#ifndef __PListS_h
#include "PListS.h"
#endif

#ifndef __AutoFree_h
#include "AutoFree.h"
#endif

#ifndef __RCX_Constants_h
#include "RCX_Constants.h"
#endif


class Symbol;
class BlockStmt;
class Bytecode;
class Program;
class VarPool;

class Fragment : public PLinkS<Fragment>, public AutoFree
{
public:
	enum
	{
		kNoTaskID = -1,
		kMultiTaskID = -2
	};

				Fragment(RCX_FragmentType type, Symbol *name, BlockStmt *body);
				~Fragment();

	int					GetNumber()	const	{ return fNumber; }
	Symbol*				GetName() const		{ return fName; }
	RCX_FragmentType	GetType() const		{ return fType; }
	int					GetTaskID() const	{ return fTaskID; }
	BlockStmt*			GetBody()			{ return fBody; }
	
	void				SetNumber(int n)	{ fNumber = n; }
	void		AssignTaskID(int n);	// only affects kRCX_SubFragment types
	
	void		Emit(Bytecode &b);
	bool		Check(VarPool *v);
	
private:
	RCX_FragmentType	fType;
	Symbol*		fName;
	BlockStmt*	fBody;
	int			fNumber;
	int			fTaskID;
};

#endif
